@@ -3,7 +3,7 @@
 """
 The MIT License (MIT)
 
-Copyright (c) 2015-2019 Rapptz
+Copyright (c) 2015-2020 Rapptz
 
 Permission is hereby granted, free of charge, to any person obtaining a
 copy of this software and associated documentation files (the "Software"),
@@ -58,6 +58,8 @@ def _get_from_guilds(bot, getter, argument):
             return result
     return result
 
+_utils_get = discord.utils.get
+
 class Converter:
     """The base class of custom converters that require the :class:`.Context`
     to be passed to be useful.
@@ -124,7 +126,7 @@ class MemberConverter(IDConverter):
         else:
             user_id = int(match.group(1))
             if guild:
-                result = guild.get_member(user_id)
+                result = guild.get_member(user_id) or _utils_get(ctx.message.mentions, id=user_id)
             else:
                 result = _get_from_guilds(bot, 'get_member', user_id)
 
@@ -152,10 +154,16 @@ class UserConverter(IDConverter):
 
         if match is not None:
             user_id = int(match.group(1))
-            result = ctx.bot.get_user(user_id)
+            result = ctx.bot.get_user(user_id) or _utils_get(ctx.message.mentions, id=user_id)
         else:
             arg = argument
-            # check for discriminator if it exists
+
+            # Remove the '@' character if this is the first character from the argument
+            if arg[0] == '@':
+                # Remove first character
+                arg = arg[1:]
+
+            # check for discriminator if it exists,
             if len(arg) > 5 and arg[-5] == '#':
                 discrim = arg[-4:]
                 name = arg[:-5]
@@ -171,11 +179,10 @@ class UserConverter(IDConverter):
             raise BadArgument('User "{}" not found'.format(argument))
 
         return result
-
 class MessageConverter(Converter):
     """Converts to a :class:`discord.Message`.
 
-    .. versionadded:: 1.1.0
+    .. versionadded:: 1.1
 
     The lookup strategy is as follows (in order):
 
